<?php

namespace Ems\Controllers;

use Ems\Services\MarcacaoService;
use Ems\Services\OcorrenciaService;
use Ems\Services\EmailService;
use Ems\Exception\CustomException;
use Ems\Services\VinculoService;
use Ems\Util\ExpectedException;
use Ems\Util\Time;
use Ems\Util\Utils;
use Ems\Logic\Offline\OfflineTimeRegister;
use Util\{ DateUtil, ControladorAlerta, Environment };

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\{ Error, Message };

class MarcacaoController {
    /** @var MarcacaoService */
    protected $marcacaoService;

    public function __construct(
        Environment $environment,
        MarcacaoService $marcacaoService,
        OcorrenciaService $ocorrenciaService,
        VinculoService $vinculoService
    ) {
        $this->environment          = $environment;
        $this->marcacaoService      = $marcacaoService;
        $this->ocorrenciaService    = $ocorrenciaService;
        $this->vinculoService       = $vinculoService;
    }

    public function getVinculosGestor(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrEstrutura   = (int) $row['NRESTRUTURAM'];

            $nrPeriodo     = isset($row['NRPERIODOAPURACAO']) ?
                (int) $row['NRPERIODOAPURACAO'] :
                -1;

            $nrTipoVinculo = (int) $row['NRTIPOVINCULOM'];

            $nrVinculomGestor = isset($row['NRVINCULOM_GESTOR']) ?
                (int) $row['NRVINCULOM_GESTOR'] :
                null;

            $dtMesCompetencia = isset($row['DTMESCOMPETENC']) ?
                (string) $row['DTMESCOMPETENC'] :
            null;

            $vinculos = $this->vinculoService->getVinculosGestor($nrEstrutura, $nrPeriodo, $nrTipoVinculo, $nrVinculomGestor, $dtMesCompetencia);

            $response->addDataSet(new DataSet('getVinculosGestor', $vinculos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getVinculosDoGestorAndPerfilRegPntMlt(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrEstrutura   = (int) $row['NRESTRUTURAM'];

            $nrPeriodo     = isset($row['NRPERIODOAPURACAO']) ?
                (int) $row['NRPERIODOAPURACAO'] :
                -1;

            $nrTipoVinculo = (int) $row['NRTIPOVINCULOM'];

            $nrVinculomGestor = isset($row['NRVINCULOM_GESTOR']) ?
                (int) $row['NRVINCULOM_GESTOR'] :
                null;

            $vinculos = $this->vinculoService->getVinculosDoGestorAndPerfilRegPntMlt($nrEstrutura, $nrPeriodo, $nrTipoVinculo, $nrVinculomGestor);

            $response->addDataSet(new DataSet('getVinculosDoGestorAndPerfilRegPntMlt', $vinculos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function setMarcacao(
        $nrvinculom,
        $cdoperador,
        $dslatitude,
        $dslongitude,
        $marcacaoAlmoco = false,
        $photoContent = null,
        $nrColetor = null
    ): array {
        try {
            $nrorg = $this->environment->getNrorg();

            $setMarcacaoStatus = $this->marcacaoService->setMarcacao(
                $nrvinculom,
                $nrorg,
                $cdoperador,
                $dslatitude,
                $dslongitude,
                $photoContent,
                $nrColetor
            );

            $nrmarcacaoponto = $setMarcacaoStatus['nrmarcacaoponto'];

            if ($marcacaoAlmoco) {
                $this->ocorrenciaService->setOcorrenciaAlmoco($nrvinculom, $nrorg, $nrmarcacaoponto, $cdoperador);
            }

            return $setMarcacaoStatus;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function registraMarcacaoIntervalo(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrorg = $this->environment->getNrorg();

            /**
             * Bloqueio temporário
             * Cliente: 2632 MALACARNE REFEICOES COLETIVAS
             * @author Marcos.Araujo
             */
            if($nrorg == 2632 && $this->environment->isRequestFromPortalFuncionario()) {
                throw new \Exception('O registro de ponto pelo Portal Funcionário não está disponível para essa organização.');
            };

            $setMarcacaoStatus = $this->setMarcacao(
                $row['NRVINCULOM'],
                $row['CDOPERADOR'],
                $row['DSLATITUDE'],
                $row['DSLONGITUDE'],
                false,
                $row['DSPATH'],
                $row['NRCOLETOR'] ?? null
            );
            $response->addDataSet(new DataSet('setMarcacaoIntervalo', $setMarcacaoStatus));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }

    public function registraMarcacaoAlmoco(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrorg = $this->environment->getNrorg();

            /**
             * Bloqueio temporário
             * Cliente: 2632 MALACARNE REFEICOES COLETIVAS
             * @author Marcos.Araujo
             */
            if($nrorg == 2632 && $this->environment->isRequestFromPortalFuncionario()) {
                throw new \Exception('O registro de ponto pelo Portal Funcionário não está disponível para essa organização.');
            };

            $setMarcacaoStatus = $this->setMarcacao(
                $row['NRVINCULOM'],
                $row['CDOPERADOR'],
                $row['DSLATITUDE'],
                $row['DSLONGITUDE'],
                true,
                $row['DSPATH'],
                $row['NRCOLETOR']
            );
            $response->addDataSet(new DataSet('setMarcacaoAlmoco', $setMarcacaoStatus));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }

    public function registraOcorrenciaPonto(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();

            $nrorg = $this->environment->getNrorg();

            /**
             * Bloqueio temporário
             * Cliente: 2632 MALACARNE REFEICOES COLETIVAS
             * @author Marcos.Araujo
             */
            if($nrorg == 2632 && $this->environment->isRequestFromPortalFuncionario()) {
                throw new \Exception('O registro de ponto pelo Portal Funcionário não está disponível para essa organização.');
            };

            $setMarcacaoStatus = $this->marcacaoService->setMarcacao(
                $row['NRVINCULOM'],
                $row['NRORG'],
                $row['CDOPERADOR'],
                $row['DSLATITUDE'],
                $row['DSLONGITUDE'],
                $row['DSPATH'],
                $row['NRCOLETOR']
            );

            $row['NRMARCACAOPONTO'] = $setMarcacaoStatus['nrmarcacaoponto'];

            $success = $this->ocorrenciaService->setOcorrenciaFreq($row);
            $setMarcacaoStatus['success'] = $success;

            $response->addDataSet(new DataSet('setOcorrenciaPonto', $setMarcacaoStatus));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function validaMarcacaoDistancia(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $validacoes = $this->marcacaoService->getValidacaoMarcacaoDistancia(
                $row['NRVINCULOM'],
                $this->environment->getNrorg(),
                $row['DSLATITUDE'],
                $row['DSLONGITUDE']
            );
            $response->addDataSet(new DataSet('validacoesMarcacao', array('validacoesMarcacao' => $validacoes)));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR));
        }
    }

    public function getParesMarcacoesDia(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int) $row['NRVINCULOM'];
            $dia       = $row['DIA'];

            $paresMarcacoes = $this->marcacaoService->
                getParesMarcacoesDia($nrVinculo, $dia);

            $response->addDataSet(new DataSet('getParesMarcacoesDia', $paresMarcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function removeParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int) $row['NRVINCULOM'];
            $nrMarcacaoEntrada = (int) $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int) $row['NRMARCACAOSAIDA'];

            $marcacoesRemovidas = $this->marcacaoService->
                removeParMarcacao($nrVinculo, $nrMarcacaoEntrada, $nrMarcacaoSaida);

            $dataSet = [
                'marcacoesRemovidas' => $marcacoesRemovidas
            ];

            $response->addDataSet(new DataSet('removeParMarcacao', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function removeParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int)     $row['NRVINCULOM'];
            $hrMarcacaoEntrada  = (string)  $row['HRENTRADA'];
            $hrMarcacaoSaida    = (string)  $row['HRSAIDA'];
            $date               = (array)   $row['DATE'];
            $idEntrada          = $row['DIA_INICIO'] ?? 'FC';
            $idSaida            = $row['DIA_FIM'] ?? 'FC';

            $marcacoesRemovidas = $this->marcacaoService->removeParesMarcacao(
                $nrVinculo, $hrMarcacaoEntrada, $hrMarcacaoSaida,
                $date, $idEntrada, $idSaida
            );

            $response->addDataSet(new DataSet('marcacoesRemovidas', $marcacoesRemovidas));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function removeMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int)     $row['NRVINCULOM'];
            $nrMarcacaoponto    = (string)  $row['NRMARCACAOPONTO'];

            $marcacoesRemovidas = $this->marcacaoService->
                removeMarcacao($nrVinculo, $nrMarcacaoponto);

            $response->addDataSet(new DataSet('marcacoesRemovidas', array($marcacoesRemovidas)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function editParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int) $row['NRVINCULOM'];
            $nrMarcacaoEntrada = (int) $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int) $row['NRMARCACAOSAIDA'];
            $dia               = (string) $row['DIA'];
            $hrEntrada         = (string) $row['HRENTRADA'];
            $hrSaida           = (string) $row['HRSAIDA'];
            $idDiaInicio       = (string) $row['DIA_INICIO'];
            $idDiaFim          = (string) $row['DIA_FIM'];

            $nrMarcacoes = $this->marcacaoService->editParMarcacao(
                $nrVinculo,
                $nrMarcacaoEntrada,
                $nrMarcacaoSaida,
                $dia,
                $hrEntrada,
                $hrSaida,
                $idDiaInicio,
                $idDiaFim
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('editParMarcacao', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function editParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrVinculo          = (int)     $row['NRVINCULOM'];
            $hrMarcacaoEntrada  = (string)  $row['HRENTRADA'];
            $hrMarcacaoSaida    = (string)  $row['HRSAIDA'];
            $dia                = (array)   $row['DATE'];
            $novaHrEntrada      = (string)  $row['NOVAHRENTRADA'];
            $novaHrSaida        = (string)  $row['NOVAHRSAIDA'];
            $idEntrada          = $row['IDENTRADA'] ?? 'FC';
            $idSaida            = $row['IDSAIDA'] ?? 'FC';
            $novoIdEntrada      = $row['NOVOIDENTRADA'] ?? $idEntrada;
            $novoIdSaida        = $row['NOVOIDSAIDA'] ?? $idSaida;

            $nrColetor          = $row['NRCOLETOR'] ?? null;

            $nrMarcacoes = $this->marcacaoService->editParesMarcacao(
                $nrVinculo,
                $hrMarcacaoEntrada,
                $hrMarcacaoSaida,
                $novaHrEntrada,
                $novaHrSaida,
                $dia,
                $idEntrada,
                $idSaida,
                $novoIdEntrada,
                $novoIdSaida,
                $nrColetor
            );

            $response->addDataSet(new DataSet('editParesMarcacao', $nrMarcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function saveParMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrorg = $this->environment->getNrorg();

            /**
             * Bloqueio temporário
             * Cliente: 2632 MALACARNE REFEICOES COLETIVAS
             * @author Marcos.Araujo
             */
            if($nrorg == 2632 && $this->environment->isRequestFromPortalFuncionario()) {
                throw new \Exception('O registro de ponto pelo Portal Funcionário não está disponível para essa organização.');
            };

            $nrVinculo         = (int) $row['NRVINCULOM'];
            $dia               = (string) $row['DIA'];
            $hrEntrada         = (string) $row['HRENTRADA'];
            $hrSaida           = (string) $row['HRSAIDA'];
            $idDiaInicio       = (string) $row['DIA_INICIO'];
            $idDiaFim          = (string) $row['DIA_FIM'];
            $coletor           = (string) ($row['NRCOLETOR'] ?? null);
            $nrMotivoJust      = empty($row['NRMOTIVOJUST']) ? null : ((int) $row['NRMOTIVOJUST']);
            $justificativa     = (string) ($row['JUSTIFICATIVA'] ?? null);
            $file              = (string) ($row['FILE'] ?? null);

            $nrMarcacoes = $this->marcacaoService->saveParMarcacao(
                $nrVinculo, $dia, $hrEntrada,
                $hrSaida, $idDiaInicio, $idDiaFim,
                $coletor, $nrMotivoJust, $justificativa, 
                $file
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('saveParMarcacao', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function saveParesMarcacao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();

            $nrorg = $this->environment->getNrorg();

            /**
             * Bloqueio temporário
             * Cliente: 2632 MALACARNE REFEICOES COLETIVAS
             * @author Marcos.Araujo
             */
            if($nrorg == 2632 && $this->environment->isRequestFromPortalFuncionario()) {
                throw new \Exception('O registro de ponto pelo Portal Funcionário não está disponível para essa organização.');
            };

            $nrVinculo      = (int)     $row['NRVINCULOM'];
            $cdOperador     =           $row['CDOPERADOR'] ?? null;
            $dias           = (array)   $row['DIA'];

            $hrEntrada      = (string)  $row['HRENTRADA'];
            $hrSaida        = (string)  $row['HRSAIDA'];

            $nrColetor      = $row['NRCOLETOR'] ?? null;

            $idDiaInicio    = $row['DIA_INICIO'] ?? 'FC';
            $idDiaFim       = $row['DIA_FIM'] ?? 'FC';
            
            $nrMotivoJust   = empty($row['NRMOTIVOJUST']) ? null : ((int) $row['NRMOTIVOJUST']);
            $justificativa  = (string) ($row['JUSTIFICATIVA'] ?? null);
            $file           = (string) ($row['FILE'] ?? null);

            $nrMarcacoes = $this->marcacaoService->saveParesMarcacao(
                $nrVinculo, $dias, $hrEntrada,
                $hrSaida, $idDiaInicio, $idDiaFim,
                $nrColetor, $nrMotivoJust, $justificativa,
                $file,
                $cdOperador
            );

            $dataSet = [
                'nrMarcacoes' => $nrMarcacoes
            ];

            $response->addDataSet(new DataSet('saveParesMarcacao', $dataSet));
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function getMarcacoesFotos(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $filesData = $this->marcacaoService->getMarcacoesFotos(
                $nrVinculo, $date
            );

            $response->addDataSet(new DataSet('getMarcacoesFotos', $filesData));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function setEntradasSaidasFixos(
        Request\Row $request,
        Response $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculom     = (int)     $row['NRVINCULOM'];
            $cdOperador     = (int)     $row['CDOPERADOR'];
            $marcacoes      = (array)   $row['ROWS'];
            $nrColetor      = $row['NRCOLETOR'] ?? null;

            $alteracoesRealizadasComSucesso = $this->marcacaoService->alteraEntradasSaidasFixadosComProjecao($nrVinculom, $cdOperador, $marcacoes, $nrColetor);

            $responseData = array('success' => $alteracoesRealizadasComSucesso);

            $response->addDataSet(new DataSet('setEntradasSaidasFixos', $responseData));
        } catch (ExpectedException $e) {
            $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function fixaEntradaSaida(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo         = (int)    $row['NRVINCULOM'];
            $date              = (string) $row['DATE'];
            $nrMarcacaoEntrada = (int)    $row['NRMARCACAOENTRADA'];
            $nrMarcacaoSaida   = (int)    $row['NRMARCACAOSAIDA'];

            $status = $this->marcacaoService->fixaEntradaSaida(
                $nrVinculo, $date,
                $nrMarcacaoEntrada, $nrMarcacaoSaida
            );

            $response->addDataSet(new DataSet('fixaEntradaSaida', $status));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace()));
        }
    }

    public function getMarcacoesPontoDiaOpcoesFixarMarcacao(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $marcacoes = $this->marcacaoService->getMarcacoesPontoDiaOpcoesFixarMarcacao(
                $nrVinculo,
                $date
            );

            $response->addDataSet(new DataSet('getMarcacoesPontoDiaOpcoesFixarMarcacao', $marcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function reverterEntradaSaida(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $date      = (string) $row['DATE'];

            $success = $this->marcacaoService->reverterEntradaSaida(
                $nrVinculo,
                $date
            );

            $dataset = [
                'success' => $success
            ];

            $response->addDataSet(new DataSet('reverterEntradaSaida', $dataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function getDetalheAcompanhamento(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow();

            $nrVinculo = (int)    $row['NRVINCULOM'];
            $dates     = (array)  $row['DATAAPURACAO'];

            $marcacoes = $this->marcacaoService->getDetalheAcompanhamento(
                $nrVinculo, $dates
            );

            $response->addDataSet(new DataSet('getDetalheAcompanhamento', $marcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function sincronizarMarcacoesOffline(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $offlineTimeRegister = OfflineTimeRegister::fromArray($row);
            $offlineTimeRegister->synchronize();
            $successDataset = array('success' => true);
            $response->addDataSet(new DataSet('sincronizarMarcacoesOffline', $successDataset));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function isMomentoForaHorario(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $nrVinculo = (int) $row['NRVINCULOM'];
            $isMomentoForaHorario = $this->marcacaoService->isMomentoForaHorario($nrVinculo);
            $response->addDataSet(new DataSet('isMomentoForaHorario', [$isMomentoForaHorario]));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }
    
    public function alertaRegistrosFaltantesUltimaSemana(
        Request\Row $request,
        Response    $response
    ) {
        try {
            $row = $request->getRow()->getArrayCopy();
            $params = [
                'NRVINCULOM_LIST' => $row['NRVINCULOM_LIST'] ?? [],
                'NRESTRUTLEGAL_LIST' => $row['NRESTRUTLEGAL_LIST'] ?? [],
                'NRESTRUTGEREN_LIST' => $row['NRESTRUTGEREN_LIST'] ?? [],
                'NRTIPOVINCULOM_LIST' => $row['NRTIPOVINCULOM_LIST'] ?? [],
                'NRCOLETOR_LIST' => $row['NRCOLETOR_LIST'] ?? [],
                'DTINICIOVERIFICACAO' => $row['DTINICIOVERIFICACAO'] ?? null,
                'DTFINALVERIFICACAO' => $row['DTFINALVERIFICACAO'] ?? null,
            ];
            
            Utils::getInstanceByClassName(EmailService::class)->sendEmailAlertaRegistrosFaltantesUltimaSemana(
                $params
            );
            
            $response->addDataSet(new DataSet('info', ControladorAlerta::getMensagensProcessoAtual()));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTraceAsString(), $e));
        }
    }

    public function sincronizarMarcacoesIntervaloOffline(Request\Row $request, Response $response)
    {
        try {
            $rows = $request->getRow();

            $result = $this->marcacaoService->sincronizarMarcacoesIntervaloOffline($rows);

            $response->addDataSet(new DataSet('sincronizarMarcacoesIntervaloOffline', $result));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }
    public function limparCacheMarcacoesOffline(Request\Row $request, Response $response)
    {
        try {
            $rows = $request->getRow();

            $marcacoes = $this->marcacaoService->limparCacheMarcacoesOffline($rows);

            $response->addDataSet(new DataSet('limparCacheMarcacoesOffline', $marcacoes));
        } catch (\Exception $e) {
            $variables = null;
            if ($e instanceof CustomException) {
                $variables = $e->getVariables();
            }
            $response->addMessage(new Response\Message($e->getMessage(), Response\Message::TYPE_ERROR, null, $variables));
        }
    }

    public function getMarcacoesDia(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            
            // Se receber esta variável, verifica se existem marcações para cada em relação a quantidade de dias para verificação
            // No caso de ocorrências, todos os dias em qual a ocorrência será adicionada devem ser verificados
            if(!empty($row['QTDDIAOCORRE'])) {
                $marc = [];
                $dtMarcacao = DateUtil::getDataDeString($row['DTINICIOPROGOCOR'], DateUtil::FORMATO_BRASILEIRO, true);
                
                for($i = 0; $i < $row['QTDDIAOCORRE']; $i++) {
                    $dt = DateUtil::adicionaIntervalo($dtMarcacao, $i);
                    $marc[] = $this->marcacaoService->getMarcacoesDia($row['NRVINCULOM'], $dt->format('d/m/Y'));
                }
                
                $marcacaoInexistente = false;
                $marcacaoAux = [];
                foreach($marc as $marcacao) {
                    if(empty($marcacao)) {
                        $marcacaoInexistente = true;
                        continue;
                    } else {
                        $marcacaoAux = $marcacao;
                    }
                }

                if(!$marcacaoInexistente) {
                    $marcacoes = $marcacaoAux;
                } else {
                    $marcacoes = [];
                }
            } else {
                $marcacoes = $this->marcacaoService->getMarcacoesDia($row['NRVINCULOM'], $row['DTINICIOPROGOCOR']);
            }

            $response->addDataSet(new DataSet('getMarcacoesDia', $marcacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }

    public function getVinculosGestorSimples(Request\Row $request, Response $response)
    {
        try {
            $row = $request->getRow();

            $nrEstrutura   = (int) $row['NRESTRUTURAM'];

            $nrPeriodo     = isset($row['NRPERIODOAPURACAO']) ?
                (int) $row['NRPERIODOAPURACAO'] :
                -1;

            $nrVinculomGestor = isset($row['NRVINCULOM_GESTOR']) ?
                (int) $row['NRVINCULOM_GESTOR'] :
                null;

            $dtMesCompetencia = isset($row['DTMESCOMPETENC']) ?
                (string) $row['DTMESCOMPETENC'] :
            null;

            $vinculos = $this->vinculoService->getVinculosGestorSimples($nrEstrutura, $nrPeriodo, 1, $nrVinculomGestor, $dtMesCompetencia);

            $response->addDataSet(new DataSet('getVinculosGestorSimples', $vinculos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode(), $e->getTrace(), $e));
        }
    }
}

